@@ -3,7 +3,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -393,8 +393,6 @@ static NSImage *colorIcon(Color color, int pixels = 20)
 }
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 - (NSTouchBar *) makeTouchBar
 {
   NSTouchBar *bar = [[NSTouchBar alloc] init];
@@ -436,7 +434,6 @@ static NSImage *colorIcon(Color color, int pixels = 20)
   }
   return nil;
 }
-#pragma clang diagnostic pop
 #endif
 
 - (void) ipeTouchBar:(id) sender
@@ -640,6 +637,11 @@ AppUi::AppUi(lua_State *L0, int model)
   [tb setSizeMode:NSToolbarSizeModeSmall];
   [iWindow setToolbar:tb];
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 110000
+  if (@available(macOS 11, *))
+    [iWindow setToolbarStyle:NSWindowToolbarStyleExpanded];
+#endif
+
   [iWindow setWindowController:iDelegate];
 
   iContent = [[NSView alloc] initWithFrame:contentRect];
@@ -1330,10 +1332,8 @@ void AppUi::setActionState(const char *name, bool value)
     }
     // update touch bar if it exists
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-    if ([iDelegate respondsToSelector:@selector(touchBar)]) {
-      id tbar = [iDelegate performSelector:@selector(touchBar)];
+    if (@available(macOS 10.12, *)) {
+      id tbar = [iDelegate touchBar];
       if (tbar != nil) {
 	auto bar = (NSTouchBar *) tbar;
 	auto item = [bar itemForIdentifier:a];
@@ -1343,7 +1343,6 @@ void AppUi::setActionState(const char *name, bool value)
 	}
       }
     }
-#pragma clang diagnostic push
 #endif
   } else if ([a isEqualToString:@"shift_key"]) {
     int mod = s.state ? CanvasBase::EShift : 0;
