@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -483,12 +483,16 @@ function MODEL:runLatex()
   self.ui:type3Font() -- reset flag in canvas
   self.type3_font = false
   local success, errmsg, result, log
-  ipeui.waitDialog(self.ui:win(),
-		   function ()
-		     success, errmsg, result, log = self.doc:runLatex(self.file_name, true) -- async
-		   end, "Compiling Latex")
-  if success and success ~= true then
-    self.doc:completeLatexRun(success) -- on ui thread
+  if prefs.freeze_in_latex then
+    success, errmsg, result, log = self.doc:runLatex(self.file_name) -- sync
+  else
+    ipeui.waitDialog(self.ui:win(),
+		     function ()
+		       success, errmsg, result, log = self.doc:runLatex(self.file_name, true) -- async
+		     end, "Compiling Latex")
+    if success and success ~= true then
+      self.doc:completeLatexRun(success) -- on ui thread
+    end
   end
   if success then
     self.ui:setResources(self.doc)
@@ -498,7 +502,7 @@ function MODEL:runLatex()
     self:latexErrorBox(log)
     return false
   elseif result == "runlatex" then
-    local urlFile = config.latexdir .. prefs.fsep .. "url.txt"
+    local urlFile = config.latexdir .. prefs.fsep .. "url1.txt"
     if ipe.fileExists(urlFile) then
       self:warning("Ipe did not succeed to run Latex to process your text objects.",
 		   "Ipe tried to use Latex in the cloud.\n\n"
