@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -271,8 +271,10 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
   }
   Buffer alphaChannel;
   uint8_t *alpha = nullptr;
+  uint32_t colorKey = 0;
   const PdfDict *smask = r->getDict(d, "SMask");
   if (smask) {
+    // ipeDebug("Mask: %s", smask->dictRepr().z());
     const PdfObj *mcs = smask->get("ColorSpace");
     if (!mcs || !mcs->name() || mcs->name()->value() != "DeviceGray"
 	|| smask->getInteger("BitsPerComponent") != 8) {
@@ -282,6 +284,20 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
       if (alphaChannel.size() == width * height)
 	alpha = (uint8_t *) alphaChannel.data();
     }
+  } else {
+    std::vector<double> ckv;
+    if (d->getNumberArray("Mask", nullptr, ckv) &&
+	size(ckv) == 2 * components) {
+      if (components == 3) {
+	uint8_t r = uint8_t(ckv[0]);
+	uint8_t g = uint8_t(ckv[2]);
+	uint8_t b = uint8_t(ckv[4]);
+	colorKey = 0xff000000 | (r << 16) | (g << 8) | b;
+      } else {
+	uint8_t g = uint8_t(ckv[0]);
+	colorKey = 0xff000000 | (g << 16) | (g << 8) | g;
+      }
+    }
   }
   Buffer stream = d->inflate();
   Buffer pixels(4 * width * height);
@@ -300,11 +316,14 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
 	if (pngPrediction)
 	  ++p; // skip predictor byte
 	for (int x = 0; x < width; ++x) {
-	  uint8_t r = uint8_t(*p++);
-	  uint8_t g = uint8_t(*p++);
-	  uint8_t b = uint8_t(*p++);
 	  uint8_t a = alpha ? *alpha++ : 0xff;
+	  // premultiply with alpha
+	  uint8_t r = uint8_t(*p++) * a / 0xff;
+	  uint8_t g = uint8_t(*p++) * a / 0xff;
+	  uint8_t b = uint8_t(*p++) * a / 0xff;
 	  uint32_t pixel = (a << 24) | (r << 16) | (g << 8) | b;
+	  if (pixel == colorKey)
+	    pixel = 0x0;
 	  *q++ = pixel;
 	}
       }
@@ -313,9 +332,11 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
 	if (pngPrediction)
 	  ++p; // skip predictor byte
 	for (int x = 0; x < width; ++x) {
-	  uint8_t r = uint8_t(*p++);
 	  uint8_t a = alpha ? *alpha++ : 0xff;
+	  uint8_t r = uint8_t(*p++) * a / 0xff;
 	  uint32_t pixel = (a << 24) | (r << 16) | (r << 8) | r;
+	  if (pixel == colorKey)
+	    pixel = 0x0;
 	  *q++ = pixel;
 	}
       }
@@ -330,7 +351,7 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
   Matrix tf = Matrix(1.0 / width, 0.0, 0.0, -1.0 / height, 0.0, 1.0);
   cairoTransform(cr, tf);
   cairo_set_source_surface(cr, image, 0, 0);
-  cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
+  cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_GOOD);
   cairo_paint_with_alpha(cr, opacity);
   cairo_restore(cr);
 }
@@ -465,14 +486,15 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_pattern_set_matrix(p, &m);
 
       cairo_set_source(iCairo, p);
+      cairo_pattern_destroy(p);  // pass ownership to iCairo
 
       if (mode == EStrokedAndFilled)
 	cairo_fill_preserve(iCairo);
       else
 	cairo_fill(iCairo);
 
-      cairo_pattern_destroy(p);
-
+      // release pattern
+      cairo_set_source_rgb(iCairo, 0.0, 0.0, 0.0);
     } else {
       // tiling
 
@@ -492,6 +514,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_fill(cc);
       cairo_destroy(cc);
       cairo_pattern_t *p = cairo_pattern_create_for_surface(s);
+      cairo_surface_destroy(s); // pass ownership to pattern
       cairo_pattern_set_extend(p, CAIRO_EXTEND_REPEAT);
 
       cairo_matrix_t m;
@@ -500,14 +523,15 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_pattern_set_matrix(p, &m);
 
       cairo_set_source(iCairo, p);
+      cairo_pattern_destroy(p); // pass ownership to iCairo
 
       if (mode == EStrokedAndFilled)
 	cairo_fill_preserve(iCairo);
       else
 	cairo_fill(iCairo);
 
-      cairo_pattern_destroy(p);
-      cairo_surface_destroy(s);
+      // release pattern so pattern and surface are destroyed
+      cairo_set_source_rgb(iCairo, 0.0, 0.0, 0.0);
     }
   }
 
@@ -580,7 +604,7 @@ void CairoPainter::doDrawBitmap(Bitmap bitmap)
 				0.0, 1.0);
   cairoTransform(iCairo, tf);
   cairo_set_source_surface(iCairo, image, 0, 0);
-  cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_FAST);
+  cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_GOOD);
   cairo_paint_with_alpha(iCairo, opacity().toDouble());
   cairo_restore(iCairo);
 }
