@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -130,7 +130,7 @@ const wchar_t Canvas::className[] = L"ipeCanvasWindowClass";
 
 void Canvas::invalidate()
 {
-  InvalidateRect(hwnd, NULL, FALSE);
+  InvalidateRect(hwnd, nullptr, FALSE);
 }
 
 void Canvas::invalidate(int x, int y, int w, int h)
@@ -240,7 +240,7 @@ void Canvas::wndPaint()
     drawFifi(cr);
 
   if (iPage) {
-    CairoPainter cp(iCascade, iFonts, cr, iZoom, false);
+    CairoPainter cp(iCascade, iFonts.get(), cr, iZoom, false);
     cp.transform(canvasTfm());
     cp.pushMatrix();
     drawTool(cp);
@@ -373,7 +373,7 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
   switch (message) {
   case WM_CREATE:
     {
-      assert(canvas == 0);
+      assert(canvas == nullptr);
       LPCREATESTRUCT p = (LPCREATESTRUCT) lParam;
       canvas = (Canvas *) p->lpCreateParams;
       canvas->hwnd = hwnd;
@@ -482,7 +482,7 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     ipeDebug("Windows canvas is destroyed");
     delete canvas;
     break;
-  // don't show in debug output:
+    // don't show in debug output:
   case WM_SETCURSOR:
   case WM_NCHITTEST:
     break;
@@ -495,17 +495,18 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 
 // --------------------------------------------------------------------
 
-Canvas::Canvas(HWND parent) : iHimetric(0., 0.)
+Canvas::Canvas(HWND parent, HINSTANCE hInstance) : iHimetric(0., 0.)
 {
-  HWND hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, className, L"",
-			      WS_CHILD | WS_VISIBLE,
-			      0, 0, 0, 0,
-			      parent, NULL,
-			      (HINSTANCE) GetWindowLongPtr(parent,
-							   GWLP_HINSTANCE),
-			      this);
-  if (hwnd == NULL) {
-    MessageBoxA(NULL, "Canvas creation failed!", "Error!",
+  if (hInstance == nullptr)
+    hInstance = (HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE);
+  HWND hwnd = (parent == nullptr) ?
+    CreateWindowExW(WS_EX_CLIENTEDGE, className, L"", WS_OVERLAPPEDWINDOW,
+		    CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+		    nullptr, nullptr, hInstance, this) :
+    CreateWindowExW(WS_EX_CLIENTEDGE, className, L"", WS_CHILD | WS_VISIBLE,
+		    0, 0, 0, 0, parent, nullptr, hInstance, this);
+  if (hwnd == nullptr) {
+    MessageBoxA(nullptr, "Canvas creation failed!", "Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
@@ -530,11 +531,9 @@ Canvas::Canvas(HWND parent) : iHimetric(0., 0.)
 UINT Canvas::getDpiForWindow(HWND hwnd)
 {
   if (pGetDpiForWindow) {
-    UINT nDpi = pGetDpiForWindow(hwnd) * 100 / 96;
-    ipeDebug("Updating DPI to %d", nDpi);
-    return nDpi;
+    return pGetDpiForWindow(hwnd);
   } else
-    return 100;
+    return 96;
 }
 
 void Canvas::init(HINSTANCE hInstance)
@@ -546,7 +545,7 @@ void Canvas::init(HINSTANCE hInstance)
   wc.cbClsExtra	 = 0;
   wc.cbWndExtra	 = 0;
   wc.hInstance	 = hInstance;
-  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
+  wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
   wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
   wc.lpszMenuName = NULL;  // actually child id
   wc.lpszClassName = className;
@@ -554,7 +553,7 @@ void Canvas::init(HINSTANCE hInstance)
   wc.hIconSm = NULL;
 
   if (!RegisterClassExW(&wc)) {
-    MessageBoxA(NULL, "Canvas control registration failed!", "Error!",
+    MessageBoxA(nullptr, "Canvas control registration failed!", "Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
