@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -749,10 +749,11 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
 					      Attribute::NORMAL());
       if (name.empty() || value.isSymbolic())
 	return false;
-      if (kind == EGridSize &&
-	  (!value.isNumber() || !value.number().isInteger()))
-	return false; // refuse non-integer gridsize
-      sheet.add(kind, Attribute(true, name), value);
+      if ((kind == EGridSize || kind == EAngleSize) && !value.isNumber())
+	return false;
+      // silently ignore non-integer grid size
+      if (kind != EGridSize || value.number().isInteger())
+	sheet.add(kind, Attribute(true, name), value);
     }
     tag = parseToTag();
   }
