@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -305,8 +305,10 @@ static int document_runLatex(lua_State *L)
     : (*d)->runLatex(docname, log);
 
   if (result == Document::ErrNone) {
-    // lua_pushboolean(L, true);
-    lua_pushlightuserdata(L, converter);
+    if (async)
+      lua_pushlightuserdata(L, converter);
+    else
+      lua_pushboolean(L, true);
     lua_pushnil(L);
     lua_pushnil(L);
   } else if (result == Document::ErrNoText) {
