@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -596,6 +596,9 @@ void EditorThread::run()
   if (iCommand.isEmpty()) {
     lua_call(L, 0, 0);
   } else {
+    // give wait dialog a chance to show up before we pass control to the editor
+    // this helps to make sure the editor has keyboard focus when it starts
+    QThread::msleep(100);
     int result = std::system(iCommand.toUtf8());
     (void) result;
   }
@@ -606,6 +609,7 @@ class EditorDialog : public QDialog
 public:
   EditorDialog(QString label, QWidget *parent = nullptr);
 protected:
+  void keyPressEvent(QKeyEvent *e);
   void closeEvent(QCloseEvent *ev);
 };
 
@@ -619,13 +623,25 @@ EditorDialog::EditorDialog(QString label, QWidget *parent)
   lo->addWidget(l, 0, 0);
 }
 
+void EditorDialog::keyPressEvent(QKeyEvent *e)
+{
+  // do not let Escape close the dialog
+  if (e->key() != Qt::Key_Escape)
+    QDialog::keyPressEvent(e);
+}
+
 void EditorDialog::closeEvent(QCloseEvent *ev)
 {
+  // do not let user close the dialog
   ev->ignore();
 }
 
 static int ipeui_wait(lua_State *L)
 {
+  PDialog *parent = nullptr;
+  Dialog **dlg = (Dialog **) luaL_testudata(L, 1, "Ipe.dialog");
+  if (dlg != nullptr)
+    parent = (PDialog *) *dlg;
   QString cmd;
   if (!lua_isfunction(L, 2))
     cmd = checkqstring(L, 2);
@@ -636,13 +652,20 @@ static int ipeui_wait(lua_State *L)
   EditorThread *thread = new EditorThread(L, cmd);
   EditorDialog *dialog = new EditorDialog(label);
 
-  if (cmd.isEmpty()) // call Lua function
+  if (cmd.isEmpty()) {
+    // call Lua function
     lua_pushvalue(L, 2);
-  thread->start();
-
-  if (!thread->wait(300)) {
+    thread->start();
+    if (!thread->wait(300)) {
+      dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
+      dialog->exec();
+    }
+  } else {
     dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
+    thread->start();
     dialog->exec();
+    if (parent)
+      parent->activateWindow();
   }
 
   delete dialog;
